package UtilityScripts;

use strict;
#use Getopt::Std qw ( getopts );         # module parses cmd line arguments
use Data::Dumper;
use Time::Local;
use Tie::File;
use Term::ANSIColor qw(:constants);

sub logArray ($\@){
	my $self = shift;
    my ($logfile,$whattolog) = @_;
	my $i;


	print RED, "Logging: $#$whattolog\n",RESET;
	open MYLOGFILE, ">>", "$logfile" or die("i died F\n");
	for ($i=0;$i<=$#$whattolog;$i++){
		print RED,$whattolog->[$i]," ", RESET;
		print MYLOGFILE $whattolog->[$i]," ";

		}
	close MYLOGFILE;
}

sub logHashKeys($\%){
	my $self = shift;
    my ($logfile,$whattolog) = @_;
	my $key;
	open MYLOGFILE, ">>", "$logfile" or die("i died F\n");

	#this puts our list in alphabetical order
	foreach $key (sort (keys(%$whattolog)))
	{
		print RED,$key," ", RESET;
		print MYLOGFILE $key," ";
	}
	print MYLOGFILE "\n";
	close MYLOGFILE;
}

sub logHashData($\%){
	my $self = shift;
    my ($logfile,$whattolog) = @_;
	my $key;

	open MYLOGFILE, ">>", "$logfile" or die("i died F\n");

	#this puts our list in alphabetical order
	foreach $key (sort (keys(%$whattolog)))
	{
		print MYLOGFILE $whattolog->{$key}," ";
	}

	print MYLOGFILE "\n";
	
	close MYLOGFILE;

}

sub logHash($\%){
	my $self = shift;
    my ($logfile,$whattolog) = @_;
	my $key;

	open MYLOGFILE, ">>", "$logfile" or die("i died F\n");

	#this puts our list in alphabetical order
	foreach $key (sort (keys(%$whattolog)))
	{
		print RED,$key," ", RESET;
		print MYLOGFILE $key," ";
	}
	print MYLOGFILE "\n";
	
	foreach $key (sort (keys(%$whattolog)))
	{
		print RED,$whattolog->{$key}," ", RESET;
		print MYLOGFILE $whattolog->{$key}," ";
	}
	print MYLOGFILE "\n";
	close MYLOGFILE;
}

sub logText ($$){
	my $self = shift;
    my ($logfile,$whattolog) = @_;

	print RED, "Logging: $whattolog",RESET;
	open MYLOGFILE, ">>", "$logfile" or die("i died F\n");
	print MYLOGFILE "$whattolog";
	close MYLOGFILE;
}

sub runWerami($$$){
	my $self = shift;
    my ($plotfile,$pressure,$temp) = @_;
    
	open TEMPWEREAMI, ">","tempwerami" or die ("i died F\n");
	print TEMPWEREAMI "$plotfile\n1\n$pressure $temp\n0 0\n";
	close TEMPWEREAMI;
	system("./werami < tempwerami");
	system("rm tempwerami");
}

sub getEntropy{
	#returns the entropy in a plotfile at a given T&P
    my $self = shift;
    my ($plotfile,$pressure,$temp) = @_;

	$pressure = int($pressure);
	$temp = int($temp)-.1;  #this is a hack - perplex doesn't like it if you match exactly the temp (hopefully this doesnt bring in errors)
    PerplexScripts->runWerami($plotfile,$pressure,$temp);
    
	my @input;
	# open the files here
	tie @input, 'Tie::File', "r$plotfile" or die ("i died F\n");

		foreach my $line(@input) {
			# this code block logically should go last but it's first because of our flow
			# it grabs the first data table we hit
			if ($line =~ /Entropy .* = \s*(\d*.\d*)\s*$/){
				return $1;
				}
		}
}

sub createFileStamp{
		
	my $TimeInSeconds = time();
	
	my @monthnames = (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec)); 
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday); 
	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = localtime($TimeInSeconds); 
	$year += 1900; 
	#print "This date is $monthnames[$mon]$mday$year\n";
	$min = sprintf("%02d",$min); 
	$hour = sprintf("%02d",$hour);
	$mday = sprintf("%02d",$mday);
	$mon = sprintf("%02d",$mon);
	
	#print "This time is $hour:$min:$sec\n"; 
	
	return join ("_", $monthnames[$mon],$mday,$year,$hour,$min,$sec);
}

sub runVertex{
		my $self = shift;
		my ($buildfile,$append) = @_;
		open BUILDPIPEFILE, ">", "pipe$buildfile" or die;
		print BUILDPIPEFILE "$buildfile\n";
		if ($append) {
			print BUILDPIPEFILE "n\n";
		}
		system("./vertex < pipe$buildfile");
		system("rm pipe$buildfile");
	}

sub getEntropyPath ($$$) {
	# this returns the entropy path as outlined in jamie's tutorial
	# note that most of my temp files can be done without an I/O (not that it probably matters)
	my $self = shift;
	my ($plotfile, $entropyForPath,$fileStamp)= @_;
	
	#create contor input
					open TEMPCONTOR, ">","tempcontorfile" or die ("i died A\n");
					print TEMPCONTOR "$plotfile\n2\n17\nn\nn\n200 200\nn\n";
					close TEMPCONTOR;
					
		
			# run wereami to get contor grid (cplot) make a copy of this to save time
				system("./werami < tempcontorfile");
		
			#print "$entropymin min entropy\n";
					open TEMPPATH, ">","temppathinput" or die ("i died B\n");
					print TEMPPATH "c$plotfile\n n\nn\ny\n$entropyForPath\n $entropyForPath\n 1\ny\n";
					close TEMPPATH;
			
			# run pscontor to get entropy values along path
				system("./pscontor < temppathinput");
			
			# run script to clean path
				system("./cleancontorScript -f contor.dat -n contor_cleansed$entropyForPath");
			
			#run werami to clean up path
					open TEMPCLEANPATH, ">","tempcleanuppath" or die ("i died C\n");
					print TEMPCLEANPATH "$plotfile\n4\n17\nn\n2\ncontor_cleansed$entropyForPath\n1\nn\n";  # get rid of this
					close TEMPCLEANPATH;
			
			system("./werami < tempcleanuppath");
			system("./cleanentropyScript -f c$plotfile -n entropyPath$fileStamp -v $entropyForPath -p .1");# cleansedentropy$entropymin is the path file (keep?)
				
				#clean up files
					system("rm c$plotfile");
					system("rm contor_cleansed$entropyForPath");
					system("rm contor.dat");
					system("rm temppathinput");
					system("rm c$plotfile.ps");
					system("rm tempcontorfile");
					system("rm tempcleanuppath");
			
				
		return "entropyPath$fileStamp";
}

#this function takes an entropic path and calculates the density and % of melts(G)
sub getValuesAlongPath(){
	my $self = shift;
	my ($plotfile,$phaseList, $fileStamp,$entropyPathFileName)= @_;
	
		open TEMPDATAPATH, ">","tempget_isoentropes" or die ("i died D\n");
		print TEMPDATAPATH "$plotfile\n4\n2\nn\n2\n$entropyPathFileName\n1\n";
		#loop
		my $i;
		for ($i=0;$i<=$#$phaseList;$i++){
			print TEMPDATAPATH "y\n7\n";
			print TEMPDATAPATH $phaseList->[$i], "\n";
		}
		print TEMPDATAPATH "n\n";
		close TEMPDATAPATH;
			
		# note: this won't get all the melt phases right now, just pmelt
		# run wereami to get all the variables we want
		system("./werami < tempget_isoentropes");

		
		# run cleanandorderscript to get everything in a format we want.
		system("./cleanandOrderScript -f c$plotfile -n plotready$fileStamp");
		print "plotready$fileStamp\n";
		return "plotready$fileStamp";
		#system("rm tempget_isoentropes");
}



1;  # YES, THE GODS OF PERL WANT THIS AT THE END OF A MODULE FILE. YES, YOUR EYES ARE NOT DECEIVING YOU.
